import { FC, RefObject } from "react";
import { BaseScrollOptions } from "../Utils/types";
/**
 * Object specifying hashes and the refs or refs with scroll options they point to
 */
export declare type MultiHashes<T> = Record<string, RefObject<T> | [RefObject<T>, Partial<BaseScrollOptions>]>;
export interface MultiHashProps extends Partial<BaseScrollOptions> {
    /**
     * An object specifying the hashes and the refs they point to
     *
     * Hashes can include or exclude leading "#"
     *
     * Each hash corresponds to a ref or a ref with options
     *
     * Example:
     *
     *  ```javascript
     *  const ref1 = createRef();
     *  const ref2 = createRef();
     *  const hashes = {
     *    hash1: ref1,
     *    "#hash2": [
     *      ref2,
     *      {
     *        behavior: "auto",
     *        position: "nearest",
     *        requiredPathname: ["/docs", "/contact"],
     *      },
     *    ],
     *  };
     *  return <MultiHash hashes={hashes} />;
     *  ```
     */
    hashes: MultiHashes<HTMLElement>;
}
/**
 * Component that pairs hashes with refs and scrolls to a corresponding ref when one of the hashes is present in the url
 */
declare const MultiHash: FC<MultiHashProps>;
export default MultiHash;
